/*  This file is part of the KDE project
    SPDX-FileCopyrightText: 2014 Laurent Montel <montel@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef KIO_MITS_DEBUG_H
#define KIO_MITS_DEBUG_H

#include <QLoggingCategory>
Q_DECLARE_LOGGING_CATEGORY(KIO_MITS_LOG)

#endif
